﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using QwirkleClassLibrary.Tiles;

namespace QwirkleClassLibrary.Players
{
    public class Player : INotifyPropertyChanged
    {
        public ReadOnlyCollection<Tile> Tiles => playerTiles.AsReadOnly();
        private readonly List<Tile> playerTiles = new();

        public event PropertyChangedEventHandler? PropertyChanged;
        void OnPropertyChanged([CallerMemberName] string? propertyName = null)
            => PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));

        /// <summary>
        /// Creates a player with a name
        /// </summary>
        /// <param name="name"></param>
        /// <exception cref="ArgumentNullException"></exception>
        public Player(string name)
        {
            if (name == null || string.IsNullOrEmpty(name))
            {
                throw new ArgumentNullException(name);
            }

            NameTag = name;
        }

        public string NameTag { get; }

        public bool IsPlaying { get; set; } = false;
        
        /// <summary>
        /// Adds a tile to the player's hand
        /// </summary>
        /// <param name="tile"></param>
        public void AddTileToPlayer(Tile tile)
        {
            playerTiles.Add(tile);
            OnPropertyChanged();
        }

        /// <summary>
        /// Removes a tile from the player's hand
        /// </summary>
        /// <param name="tile"></param>
        /// <returns>bool</returns>
        public bool RemoveTileToPlayer(Tile tile)
        {
            if (playerTiles.Remove(tile))
            {
                OnPropertyChanged();
                return true;
            }
            return false;
        }
    }
}
